require 'rails/generators'
require 'rails/generators/rails/app/app_generator'
require 'yaml'
require_relative '../support/servicefile'

module Betterdoc
  module Generators

    class AppBuilder < Rails::AppBuilder; end

    class AppGenerator < Rails::Generators::AppGenerator

      class << self
        def source_root(path = nil)
          Rails::Generators::AppGenerator.source_root(path)
        end

        def start(options)
          options.push("--skip-action-cable") unless options.include?("--enable-action-cable")
          options.push("--skip-javascript") unless options.include?("--enable-javascript")
          options.push("--skip-sprockets") unless options.include?("--enable-sprockets")
          options.push("--skip-yarn") unless options.include?("--enable-yarn")
          options.push("--skip-action-mailer") unless options.include?("--enable-action-mailer")
          options.push("--skip-active-storage") unless options.include?("--enable-action-storage")
          options.push("--skip-spring") unless options.include?("--enable-spring")
          options.push("--skip-coffee") unless options.include?("--enable-coffee")
          options.push("--skip-test") unless options.include?("--enable-test")
          super(options)
        end
      end

      def initialize(*args)
        super
        source_paths.unshift([File.expand_path(File.join(File.dirname(__FILE__), '..', '..', '..', 'templates'))])
      end

      def get_builder_class
        AppBuilder
      end

      def create_misc_files
        template('env.development.local', '.env.development.local')
        template('dockerignore', '.dockerignore')
        template('rspec', '.rspec')
        template('rubocop.yml', '.rubocop.yml')
        template('Dockerfile', 'Dockerfile')
        template('circleci-config', '.circleci/config.yml')
      end

      def create_servicefile_file
        create_file('Servicefile', create_servicefile.to_yaml)
      end

      def create_servicefile
        servicefile = Betterdoc::Support::Servicefile.new
        servicefile.service_name = @app_name
        servicefile.add_environment('PORT', 'The HTTP port on which the service should listen for incoming requests')
        servicefile.add_environment('SECRET_KEY_BASE', 'The Rails secret key base string')
        servicefile
      end

    end
  end
end
